@@ -29,58 +29,23 @@
 /* The icon-reading code is also in libwnck, please sync bugfixes */
 
 static void
-get_fallback_icons (MetaScreen     *screen,
-                    GdkPixbuf     **iconp,
-                    int             ideal_width,
-                    int             ideal_height,
-                    GdkPixbuf     **mini_iconp,
-                    int             ideal_mini_width,
-                    int             ideal_mini_height)
+get_fallback_icons (MetaScreen  *screen,
+                    GdkPixbuf  **iconp,
+                    int          ideal_size,
+                    GdkPixbuf  **mini_iconp,
+                    int          ideal_mini_size)
 {
   /* we don't scale, should be fixed if we ever un-hardcode the icon
    * size
    */
-  *iconp = meta_ui_get_default_window_icon (screen->ui);
-  *mini_iconp = meta_ui_get_default_mini_icon (screen->ui);
-}
-
-static gboolean
-find_largest_sizes (gulong *data,
-                    gulong  nitems,
-                    int    *width,
-                    int    *height)
-{
-  *width = 0;
-  *height = 0;
-
-  while (nitems > 0)
-    {
-      int w, h;
-
-      if (nitems < 3)
-        return FALSE; /* no space for w, h */
-
-      w = data[0];
-      h = data[1];
-
-      if (nitems < ((gulong)(w * h) + 2))
-        return FALSE; /* not enough data */
-
-      *width = MAX (w, *width);
-      *height = MAX (h, *height);
-
-      data += (w * h) + 2;
-      nitems -= (w * h) + 2;
-    }
-
-  return TRUE;
+  *iconp = meta_ui_get_default_window_icon (screen->ui, ideal_size);
+  *mini_iconp = meta_ui_get_default_mini_icon (screen->ui, ideal_mini_size);
 }
 
 static gboolean
 find_best_size (gulong  *data,
                 gulong   nitems,
-                int      ideal_width,
-                int      ideal_height,
+                int      ideal_size,
                 int     *width,
                 int     *height,
                 gulong **start)
@@ -88,20 +53,11 @@ find_best_size (gulong  *data,
   int best_w;
   int best_h;
   gulong *best_start;
-  int max_width, max_height;
 
   *width = 0;
   *height = 0;
   *start = NULL;
 
-  if (!find_largest_sizes (data, nitems, &max_width, &max_height))
-    return FALSE;
-
-  if (ideal_width < 0)
-    ideal_width = max_width;
-  if (ideal_height < 0)
-    ideal_height = max_height;
-
   best_w = 0;
   best_h = 0;
   best_start = NULL;
@@ -129,7 +85,6 @@ find_best_size (gulong  *data,
       else
         {
           /* work with averages */
-          const int ideal_size = (ideal_width + ideal_height) / 2;
           int best_size = (best_w + best_h) / 2;
           int this_size = (w + h) / 2;
 
@@ -205,18 +160,16 @@ argbdata_to_pixdata (gulong *argb_data, int len, guchar **pixdata)
 }
 
 static gboolean
-read_rgb_icon (MetaDisplay   *display,
-               Window         xwindow,
-               int            ideal_width,
-               int            ideal_height,
-               int            ideal_mini_width,
-               int            ideal_mini_height,
-               int           *width,
-               int           *height,
-               guchar       **pixdata,
-               int           *mini_width,
-               int           *mini_height,
-               guchar       **mini_pixdata)
+read_rgb_icon (MetaDisplay  *display,
+               Window        xwindow,
+               int           ideal_size,
+               int           ideal_mini_size,
+               int          *width,
+               int          *height,
+               guchar      **pixdata,
+               int          *mini_width,
+               int          *mini_height,
+               guchar      **mini_pixdata)
 {
   Atom type;
   int format;
@@ -253,17 +206,23 @@ read_rgb_icon (MetaDisplay   *display,
 
   data_as_long = (gulong *)data;
 
-  if (!find_best_size (data_as_long, nitems,
-                       ideal_width, ideal_height,
-                       &w, &h, &best))
+  if (!find_best_size (data_as_long,
+                       nitems,
+                       ideal_size,
+                       &w,
+                       &h,
+                       &best))
     {
       XFree (data);
       return FALSE;
     }
 
-  if (!find_best_size (data_as_long, nitems,
-                       ideal_mini_width, ideal_mini_height,
-                       &mini_w, &mini_h, &best_mini))
+  if (!find_best_size (data_as_long,
+                       nitems,
+                       ideal_mini_size,
+                       &mini_w,
+                       &mini_h,
+                       &best_mini))
     {
       XFree (data);
       return FALSE;
@@ -399,15 +358,13 @@ apply_mask (GdkPixbuf *pixbuf,
 }
 
 static gboolean
-try_pixmap_and_mask (MetaDisplay *display,
-                     Pixmap       src_pixmap,
-                     Pixmap       src_mask,
-                     GdkPixbuf  **iconp,
-                     int          ideal_width,
-                     int          ideal_height,
-                     GdkPixbuf  **mini_iconp,
-                     int          ideal_mini_width,
-                     int          ideal_mini_height)
+try_pixmap_and_mask (MetaDisplay  *display,
+                     Pixmap        src_pixmap,
+                     Pixmap        src_mask,
+                     GdkPixbuf   **iconp,
+                     int           ideal_size,
+                     GdkPixbuf   **mini_iconp,
+                     int           ideal_mini_size)
 {
   GdkPixbuf *unscaled = NULL;
   GdkPixbuf *mask = NULL;
@@ -457,17 +414,13 @@ try_pixmap_and_mask (MetaDisplay *display,
     {
       *iconp =
         gdk_pixbuf_scale_simple (unscaled,
-                                 ideal_width > 0 ? ideal_width :
-                                 gdk_pixbuf_get_width (unscaled),
-                                 ideal_height > 0 ? ideal_height :
-                                 gdk_pixbuf_get_height (unscaled),
+                                 ideal_size,
+                                 ideal_size,
                                  GDK_INTERP_BILINEAR);
       *mini_iconp =
         gdk_pixbuf_scale_simple (unscaled,
-                                 ideal_mini_width > 0 ? ideal_mini_width :
-                                 gdk_pixbuf_get_width (unscaled),
-                                 ideal_mini_height > 0 ? ideal_mini_height :
-                                 gdk_pixbuf_get_height (unscaled),
+                                 ideal_mini_size,
+                                 ideal_mini_size,
                                  GDK_INTERP_BILINEAR);
 
       g_object_unref (G_OBJECT (unscaled));
@@ -542,48 +495,15 @@ meta_icon_cache_init (MetaIconCache *icon_cache)
   icon_cache->origin = USING_NO_ICON;
   icon_cache->prev_pixmap = None;
   icon_cache->prev_mask = None;
-#if 0
-  icon_cache->icon = NULL;
-  icon_cache->mini_icon = NULL;
-  icon_cache->ideal_width = -1; /* won't be a legit width */
-  icon_cache->ideal_height = -1;
-  icon_cache->ideal_mini_width = -1;
-  icon_cache->ideal_mini_height = -1;
-#endif
-  icon_cache->want_fallback = TRUE;
   icon_cache->wm_hints_dirty = TRUE;
   icon_cache->kwm_win_icon_dirty = TRUE;
   icon_cache->net_wm_icon_dirty = TRUE;
 }
 
-static void
-clear_icon_cache (MetaIconCache *icon_cache,
-                  gboolean       dirty_all)
-{
-#if 0
-  if (icon_cache->icon)
-    g_object_unref (G_OBJECT (icon_cache->icon));
-  icon_cache->icon = NULL;
-
-  if (icon_cache->mini_icon)
-    g_object_unref (G_OBJECT (icon_cache->mini_icon));
-  icon_cache->mini_icon = NULL;
-#endif
-
-  icon_cache->origin = USING_NO_ICON;
-
-  if (dirty_all)
-    {
-      icon_cache->wm_hints_dirty = TRUE;
-      icon_cache->kwm_win_icon_dirty = TRUE;
-      icon_cache->net_wm_icon_dirty = TRUE;
-    }
-}
-
 void
 meta_icon_cache_free (MetaIconCache *icon_cache)
 {
-  clear_icon_cache (icon_cache, FALSE);
+  icon_cache->origin = USING_NO_ICON;
 }
 
 void
@@ -611,47 +531,19 @@ meta_icon_cache_get_icon_invalidated (MetaIconCache *icon_cache)
   else if (icon_cache->origin <= USING_NET_WM_ICON &&
            icon_cache->net_wm_icon_dirty)
     return TRUE;
-  else if (icon_cache->origin < USING_FALLBACK_ICON &&
-           icon_cache->want_fallback)
+  else if (icon_cache->origin < USING_FALLBACK_ICON)
     return TRUE;
   else if (icon_cache->origin == USING_NO_ICON)
     return TRUE;
-  else if (icon_cache->origin == USING_FALLBACK_ICON &&
-           !icon_cache->want_fallback)
-    return TRUE;
   else
     return FALSE;
 }
 
-static void
-replace_cache (MetaIconCache *icon_cache,
-               IconOrigin     origin,
-               GdkPixbuf     *new_icon,
-               GdkPixbuf     *new_mini_icon)
-{
-  clear_icon_cache (icon_cache, FALSE);
-
-  icon_cache->origin = origin;
-
-#if 0
-  if (new_icon)
-    g_object_ref (G_OBJECT (new_icon));
-
-  icon_cache->icon = new_icon;
-
-  if (new_mini_icon)
-    g_object_ref (G_OBJECT (new_mini_icon));
-
-  icon_cache->mini_icon = new_mini_icon;
-#endif
-}
-
 static GdkPixbuf*
 scaled_from_pixdata (guchar *pixdata,
                      int     w,
                      int     h,
-                     int     new_w,
-                     int     new_h)
+                     int     new_size)
 {
   GdkPixbuf *src;
   GdkPixbuf *dest;
@@ -688,9 +580,12 @@ scaled_from_pixdata (guchar *pixdata,
 	}
     }
 
-  if (w != new_w || h != new_h)
+  if (w != new_size || h != new_size)
     {
-      dest = gdk_pixbuf_scale_simple (src, new_w, new_h, GDK_INTERP_BILINEAR);
+      dest = gdk_pixbuf_scale_simple (src,
+                                      new_size,
+                                      new_size,
+                                      GDK_INTERP_BILINEAR);
 
       g_object_unref (G_OBJECT (src));
     }
@@ -709,11 +604,9 @@ meta_read_icons (MetaScreen     *screen,
                  Pixmap          wm_hints_pixmap,
                  Pixmap          wm_hints_mask,
                  GdkPixbuf     **iconp,
-                 int             ideal_width,
-                 int             ideal_height,
+                 int             ideal_size,
                  GdkPixbuf     **mini_iconp,
-                 int             ideal_mini_width,
-                 int             ideal_mini_height)
+                 int             ideal_mini_size)
 {
   guchar *pixdata;
   int w, h;
@@ -729,19 +622,6 @@ meta_read_icons (MetaScreen     *screen,
   *iconp = NULL;
   *mini_iconp = NULL;
 
-#if 0
-  if (ideal_width != icon_cache->ideal_width ||
-      ideal_height != icon_cache->ideal_height ||
-      ideal_mini_width != icon_cache->ideal_mini_width ||
-      ideal_mini_height != icon_cache->ideal_mini_height)
-    clear_icon_cache (icon_cache, TRUE);
-
-  icon_cache->ideal_width = ideal_width;
-  icon_cache->ideal_height = ideal_height;
-  icon_cache->ideal_mini_width = ideal_mini_width;
-  icon_cache->ideal_mini_height = ideal_mini_height;
-#endif
-
   if (!meta_icon_cache_get_icon_invalidated (icon_cache))
     return FALSE; /* we have no new info to use */
 
@@ -762,22 +642,30 @@ meta_read_icons (MetaScreen     *screen,
     {
       icon_cache->net_wm_icon_dirty = FALSE;
 
-      if (read_rgb_icon (screen->display, xwindow,
-                         ideal_width, ideal_height,
-                         ideal_mini_width, ideal_mini_height,
-                         &w, &h, &pixdata,
-                         &mini_w, &mini_h, &mini_pixdata))
+      if (read_rgb_icon (screen->display,
+                         xwindow,
+                         ideal_size,
+                         ideal_mini_size,
+                         &w,
+                         &h,
+                         &pixdata,
+                         &mini_w,
+                         &mini_h,
+                         &mini_pixdata))
         {
-          *iconp = scaled_from_pixdata (pixdata, w, h,
-                                        ideal_width, ideal_height);
+          *iconp = scaled_from_pixdata (pixdata,
+                                        w,
+                                        h,
+                                        ideal_size);
 
-          *mini_iconp = scaled_from_pixdata (mini_pixdata, mini_w, mini_h,
-                                             ideal_mini_width, ideal_mini_height);
+          *mini_iconp = scaled_from_pixdata (mini_pixdata,
+                                             mini_w,
+                                             mini_h,
+                                             ideal_mini_size);
 
           if (*iconp && *mini_iconp)
             {
-              replace_cache (icon_cache, USING_NET_WM_ICON,
-                             *iconp, *mini_iconp);
+              icon_cache->origin = USING_NET_WM_ICON;
 
               return TRUE;
             }
@@ -808,15 +696,17 @@ meta_read_icons (MetaScreen     *screen,
           pixmap != None)
         {
           if (try_pixmap_and_mask (screen->display,
-                                   pixmap, mask,
-                                   iconp, ideal_width, ideal_height,
-                                   mini_iconp, ideal_mini_width, ideal_mini_height))
+                                   pixmap,
+                                   mask,
+                                   iconp,
+                                   ideal_size,
+                                   mini_iconp,
+                                   ideal_mini_size))
             {
               icon_cache->prev_pixmap = pixmap;
               icon_cache->prev_mask = mask;
 
-              replace_cache (icon_cache, USING_WM_HINTS,
-                             *iconp, *mini_iconp);
+              icon_cache->origin = USING_WM_HINTS;
 
               return TRUE;
             }
@@ -834,43 +724,33 @@ meta_read_icons (MetaScreen     *screen,
            mask != icon_cache->prev_mask) &&
           pixmap != None)
         {
-          if (try_pixmap_and_mask (screen->display, pixmap, mask,
-                                   iconp, ideal_width, ideal_height,
-                                   mini_iconp, ideal_mini_width, ideal_mini_height))
+          if (try_pixmap_and_mask (screen->display,
+                                   pixmap,
+                                   mask,
+                                   iconp,
+                                   ideal_size,
+                                   mini_iconp,
+                                   ideal_mini_size))
             {
               icon_cache->prev_pixmap = pixmap;
               icon_cache->prev_mask = mask;
 
-              replace_cache (icon_cache, USING_KWM_WIN_ICON,
-                             *iconp, *mini_iconp);
+              icon_cache->origin = USING_KWM_WIN_ICON;
 
               return TRUE;
             }
         }
     }
 
-  if (icon_cache->want_fallback &&
-      icon_cache->origin < USING_FALLBACK_ICON)
+  if (icon_cache->origin < USING_FALLBACK_ICON)
     {
       get_fallback_icons (screen,
                           iconp,
-                          ideal_width,
-                          ideal_height,
+                          ideal_size,
                           mini_iconp,
-                          ideal_mini_width,
-                          ideal_mini_height);
+                          ideal_mini_size);
 
-      replace_cache (icon_cache, USING_FALLBACK_ICON,
-                     *iconp, *mini_iconp);
-
-      return TRUE;
-    }
-
-  if (!icon_cache->want_fallback &&
-      icon_cache->origin == USING_FALLBACK_ICON)
-    {
-      /* Get rid of current icon */
-      clear_icon_cache (icon_cache, FALSE);
+      icon_cache->origin = USING_FALLBACK_ICON;
 
       return TRUE;
     }
