@@ -65,8 +65,6 @@ meta_accel_label_class_init (MetaAccelLabelClass *class)
   widget_class->get_preferred_height = meta_accel_label_get_preferred_height;
   widget_class->draw = meta_accel_label_draw;
 
-  class->signal_quote1 = g_strdup ("<:");
-  class->signal_quote2 = g_strdup (":>");
   /* This is the text that should appear next to menu accelerators
    * that use the shift key. If the text on this key isn't typically
    * translated on keyboards used for your language, don't translate
@@ -129,7 +127,6 @@ meta_accel_label_class_init (MetaAccelLabelClass *class)
   class->mod_name_mod5 = g_strdup (_("Mod5"));
 
   class->mod_separator = g_strdup ("+");
-  class->accel_seperator = g_strdup (" / ");
   class->latin1_to_char = TRUE;
 }
 
@@ -340,11 +337,7 @@ meta_accel_label_update (MetaAccelLabel *accel_label)
 
   class = META_ACCEL_LABEL_GET_CLASS (accel_label);
 
-  g_free (accel_label->accel_string);
-  accel_label->accel_string = NULL;
-
-  gstring = g_string_new (accel_label->accel_string);
-  g_string_append (gstring, gstring->len ? class->accel_seperator : "   ");
+  gstring = g_string_new ("   ");
 
   if (accel_label->accel_mods & META_VIRTUAL_SHIFT_MASK)
     {
